@@ -1,5 +1,5 @@
-r"""An ISO8583 spec is a Python dict instance. It describes how each field
-needs to be encoded or decoded.
+r"""An ISO8583 spec is a Python dictionary. It describes how each field
+needs to be encoded and decoded.
 
 Fields
 ------
@@ -12,26 +12,29 @@ Supported fields:
 - **1** - Secondary Bitmap
 - **2** .. **128** - Regular fields
 
-Field Properties
-----------------
+Mandatory Field Properties
+--------------------------
 Each field defines these properties:
 
 - **data_enc** - field's data encoding type, where:
 
-  - Use ``b`` for binary-coded decimal data (e.g. an ``ABCD`` hex string
-    is encoded as ``\xAB\xCD`` 2-byte value).
+  - Use ``b`` for binary or Binary-Coded Decimal (BCD) data. For example,
+    ``ABCD`` hex string is encoded as ``\xAB\xCD`` 2-byte value. Or ``1234``
+    numeric string is encoded as ``\x12\x34`` 2-byte BCD value.
   - Otherwise, provide any valid Python encoding. For example, ``ascii`` or
     ``latin-1`` for ASCII data and ``cp500`` or similar for EBCDIC data.
+    For a list of possible encodings, see Python standard encodings:
+    https://docs.python.org/3/library/codecs.html#standard-encodings
 
 - **len_enc** - field's length encoding type. Follows the same rules as **data_enc**.
   Some fields, such as ICC data, could have binary data but ASCII length.
   This parameter does not affect fixed-length fields.
 
 - **len_type** - field's length type: fixed, LVAR, LLVAR, etc.
-  Expressed as number of bytes in field length. For example,
-  ASCII LLVAR length takes up 2 bytes (``b'00'`` - ``b'99'``).
-  BCD LLVAR length can take up only 1 byte (``b'\x00'`` - ``b'\x63'``).
-  **len_type** depends on the type of **len_enc** being used.
+  Expressed as a number of bytes in field length. For example,
+  ASCII LLVAR length takes up 2 bytes (``b'00' - b'99'``).
+  BCD LLVAR length can take up only 1 byte (``b'\x00' - b'\x99'``).
+  Therefore, **len_type** depends on the type of **len_enc** being used.
   BCD **len_enc** can fit higher length ranges in fewer bytes.
 
   +--------------+---------------------------------------+
@@ -50,17 +53,40 @@ Each field defines these properties:
   |  LLLLVAR     |          ``4`` |                ``2`` |
   +--------------+----------------+----------------------+
 
-- **max_len** - field's maximum length in bytes. For fixed fields
-  **max_len** defines the fixed length of the field.
+- **max_len** - field's maximum length in bytes or nibbles. For fixed fields
+  **max_len** defines the length of the field.
 
 - **desc** - field's description that's printed in a pretty format.
   **desc** plays no role in encoding or decoding data. It's safe to omit it
-  from the specifications. If omitted :func:`iso8583.pp` may or may not work
-  as expected.
+  from the specifications. However, if omitted :func:`iso8583.pp` may or may
+  not work as expected.
+
+Optional Field Properties
+-------------------------
+Each field may define these additional properties as needed:
+
+- **len_count** - specifies if field's length is measured
+  in bytes or nibbles (half bytes). This parameter affects **max_len**.
+
+  - Use ``bytes`` (default) to measure field length in bytes.
+  - Use ``nibbles`` to measure field length if nibbles (half bytes).
+
+- **left_pad** - specifies pad character to be added/removed on the left side
+  of an odd binary or BCD field without this character being included into
+  field length. Valid pad character is ``0-9`` for BCD fields and ``0-F``
+  for binary fields.
+
+  This option is used only when **data_enc** is set to ``b`` (binary/BCD)
+  and **len_count** is set to ``nibbles``. This option is meant for
+  specifications that require odd length binary or BCD data.
+
+- **right_pad** - same as **left_pad** but it pads on the right side.
+  Specify either **left_pad** or **right_pad**. If both are specified at
+  the same time then **left_pad** takes precedence.
 
 Sample Field Specifications
 ---------------------------
-Binary-coded decimal primary bitmap (the same applies to the secondary bitmap)::
+Binary-coded decimal primary bitmap::
 
     specification["p"] = {
         "data_enc": "b",
@@ -70,7 +96,7 @@ Binary-coded decimal primary bitmap (the same applies to the secondary bitmap)::
         "desc": "BCD bitmap, e.g. \x12\x34\x56\x78\x90\xAB\xCD\xEF"
     }
 
-Hex string primary bitmap (the same applies to the secondary bitmap)::
+Hex string primary bitmap::
 
     specification["p"] = {
         "data_enc": "ascii",
@@ -80,7 +106,7 @@ Hex string primary bitmap (the same applies to the secondary bitmap)::
         "desc": "Hex string bitmap, e.g 1234567890ABCDEF"
     }
 
-Field 2 that is a BCD fixed length field of 10 bytes::
+Field 2, a 10-byte BCD fixed length field::
 
     specification["2"] = {
         "data_enc": "b",
@@ -90,7 +116,7 @@ Field 2 that is a BCD fixed length field of 10 bytes::
         "desc": "BCD fixed field"
     }
 
-Field 3 that is an ASCII LLVAR field of maximum 20 bytes::
+Field 3, an ASCII LLVAR field of maximum 20 bytes::
 
     specification["3"] = {
         "data_enc": "ascii",
@@ -100,7 +126,7 @@ Field 3 that is an ASCII LLVAR field of maximum 20 bytes::
         "desc": "ASCII LLVAR field"
     }
 
-Field 4 that is an EBCDIC LLLVAR field of maximum 150 bytes::
+Field 4, an EBCDIC LLLVAR field of maximum 150 bytes::
 
     specification["4"] = {
         "data_enc": "cp500",
@@ -110,13 +136,38 @@ Field 4 that is an EBCDIC LLLVAR field of maximum 150 bytes::
         "desc": "EBCDIC LLLVAR field"
     }
 
+Field 5, a BCD LLVAR field measured in nibbles and left-padded with 0.
+The field is maximum 20 nibbles::
+
+    specification["5"] = {
+        "data_enc": "b",
+        "len_enc": "b",
+        "len_type": 1,
+        "len_count": "nibbles",
+        "left_pad": "0",
+        "max_len": 20,
+        "desc": "BCD LLVAR field measured in nibbles, e.g. \x03\x01\x11"
+    }
+
+Field 6, a 3-nibble BCD fixed field right-padded with 0::
+
+    specification["6"] = {
+        "data_enc": "b",
+        "len_enc": "b",
+        "len_type": 0,
+        "len_count": "nibbles",
+        "right_pad": "0",
+        "max_len": 3,
+        "desc": "BCD fixed field measured in nibbles, e.g. \x11\x10"
+    }
+
 Sample Message Specifications
 -----------------------------
 
-ASCII/BCD
-~~~~~~~~~
+ASCII/Binary
+~~~~~~~~~~~~
 
-Bitmaps, MACs, PIN, and ICC data are in BCD::
+Bitmaps, MACs, PIN, and ICC data are in binary::
 
     default = {
     "h":   {"data_enc": "ascii", "len_enc": "ascii", "len_type": 0, "max_len": 0,   "desc": "Message Header"},
@@ -393,8 +444,8 @@ All fields are in ASCII::
 
 """
 
-# ASCII/BCD
-# Bitmaps, MACs, PIN, and ICC data are in BCD
+# ASCII/Binary
+# Bitmaps, MACs, PIN, and ICC data are in binary
 default = {
     "h": {
         "data_enc": "ascii",
